LIBRARY libplot
DESCRIPTION import library for Win32 plotutils v2.4.1-1
EXPORTS

   __miCreateETandAET                  @324
   __miDrawArcs_r_internal             @2
   __miDrawLines_internal              @3
   __miDrawPoints_internal             @4
   __miDrawRectangles_internal         @5
   __miFillArcs_internal               @6
   __miFillConvexPoly                  @7
   __miFillGeneralPoly                 @8
   __miFillPolygon_internal            @9
   _miFillRectangles_internal          @10
   __miFillSppPoly                     @11
   __miFreeStorage                     @12
   __miInsertionSort                   @13
   __miPolyArc_r                       @14
   __miQuickSortSpansY                 @15
   __miStepDash                        @16
   __miUniquifyPaintedSet              @17
   __miWideDash                        @18
   __miWideLine                        @19
   __miZeroDash                        @20
   __miZeroLine                        @21
   __miZeroPolyArc_r                   @22
   __mi_xcalloc                        @23
   __mi_xmalloc                        @24
   __mi_xrealloc                       @25
   __micomputeWAET                     @26
   __miloadAET                         @27
   _a_begin_page                       @28
   _a_default_plotter                  @29
   _a_end_page                         @30
   _a_erase_page                       @31
   _a_initialize                       @32
   _a_paint_path                       @33
   _a_paint_paths                      @34
   _a_paint_point                      @35
   _a_paint_text_string                @36
   _a_set_attributes                   @37
   _a_set_fill_color                   @38
   _a_set_pen_color                    @39
   _a_terminate                        @40
   _add_arc                            @41
   _add_arc_as_bezier3                 @42
   _add_arc_as_lines                   @43
   _add_bezier2                        @44
   _add_bezier2_as_lines               @45
   _add_bezier3                        @46
   _add_bezier3_as_lines               @47
   _add_box                            @48
   _add_box_as_lines                   @49
   _add_circle                         @50
   _add_circle_as_bezier3s             @51
   _add_circle_as_ellarcs              @52
   _add_circle_as_lines                @53
   _add_closepath                      @54
   _add_ellarc                         @55
   _add_ellarc_as_bezier3              @56
   _add_ellarc_as_lines                @57
   _add_ellipse                        @58
   _add_ellipse_as_bezier3s            @59
   _add_ellipse_as_ellarcs             @60
   _add_ellipse_as_lines               @61
   _add_line                           @62
   _add_moveto                         @63
   _ai_fill_rule                       @64
   _alabel_hershey                     @65
   _angle_of_arc                       @66
   _b_begin_page                       @67
   _b_default_plotter                  @68
   _b_delete_image                     @69
   _b_draw_elliptic_arc                @70
   _b_draw_elliptic_arc_2              @71
   _b_draw_elliptic_arc_internal       @72
   _b_end_page                         @73
   _b_erase_page                       @74
   _b_initialize                       @75
   _b_maybe_output_image               @76
   _b_new_image                        @77
   _b_paint_path                       @78
   _b_paint_paths                      @79
   _b_paint_point                      @80
   _b_terminate                        @81
   _bbox_of_outbuf                     @82
   _bbox_of_outbufs                    @83
   _c_begin_page                       @84
   _c_default_plotter                  @85
   _c_end_page                         @86
   _c_erase_page                       @87
   _c_initialize                       @88
   _c_paint_marker                     @89
   _c_paint_path                       @90
   _c_paint_paths                      @91
   _c_paint_point                      @92
   _c_paint_text_string                @93
   _c_set_attributes                   @94
   _c_set_bg_color                     @95
   _c_set_fill_color                   @96
   _c_set_pen_color                    @97
   _c_terminate                        @98
   _cgm_cap_style                      @99
   _cgm_emit_color_component           @100
   _cgm_emit_command_header            @101
   _cgm_emit_command_terminator        @102
   _cgm_emit_enum                      @103
   _cgm_emit_index                     @104
   _cgm_emit_integer                   @105
   _cgm_emit_point                     @106
   _cgm_emit_points                    @107
   _cgm_emit_real_fixed_point          @108
   _cgm_emit_real_floating_point       @109
   _cgm_emit_string                    @110
   _cgm_emit_unsigned_integer          @111
   _cgm_emit_unsigned_integer_8bit     @112
   _cgm_font_id_to_ps_font             @113
   _cgm_font_properties                @114
   _cgm_join_style                     @115
   _clip_line                          @116
   _codestring_len                     @117
   _colornames                         @118
   _compute_ndc_to_device_map          @119
   _controlify                         @120
   _copy_params_to_plotter             @121
   _create_color_name_cache            @122
   _create_first_drawing_state         @123
   _default_drawstate                  @124
   _default_plotter_params             @125
   _delete_color_name_cache            @126
   _delete_first_drawing_state         @127
   _delete_outbuf                      @128
   _delete_plPath                      @129
   _draw_hershey_glyph                 @130
   _draw_hershey_penup_stroke          @131
   _draw_hershey_string                @132
   _draw_hershey_stroke                @133
   _f_begin_page                       @134
   _f_compute_line_style               @135
   _f_default_plotter                  @136
   _f_draw_arc_internal                @137
   _f_draw_box_internal                @138
   _f_draw_ellipse_internal            @139
   _f_end_page                         @140
   _f_erase_page                       @141
   _f_initialize                       @142
   _f_paint_path                       @143
   _f_paint_paths                      @144
   _f_paint_point                      @145
   _f_paint_text_string                @146
   _f_retrieve_font                    @147
   _f_set_fill_color                   @148
   _f_set_pen_color                    @149
   _f_terminate                        @150
   _fig_cap_style                      @151
   _fig_color                          @152
   _fig_join_style                     @153
   _fig_line_style                     @154
   _fig_stdcolors                      @155
   _fixed_width_spacings               @156
   _flabelwidth_hershey                @157
   _flatten_path                       @158
   _flush_plotter_outstreams           @159
   _free_params_in_plotter             @160
   _freeze_outbuf                      @161
   _g_begin_page                       @162
   _g_default_plotter                  @163
   _g_end_page                         @164
   _g_erase_page                       @165
   _g_error                            @166
   _g_flush_output                     @167
   _g_get_text_width                   @168
   _g_initialize                       @169
   _g_maybe_prepaint_segments          @170
   _g_paint_marker                     @171
   _g_paint_path                       @172
   _g_paint_paths                      @173
   _g_paint_point                      @174
   _g_paint_text_string                @175
   _g_paint_text_string_with_escapes   @176
   _g_path_is_flushable                @177
   _g_pop_state                        @178
   _g_push_state                       @179
   _g_retrieve_font                    @180
   _g_terminate                        @181
   _g_warning                          @182
   _get_default_plot_param             @183
   _get_plot_param                     @184
   _grayscale_approx                   @185
   _h_begin_page                       @186
   _h_default_plotter                  @187
   _h_end_page                         @188
   _h_erase_page                       @189
   _h_initialize                       @190
   _h_maybe_switch_from_hpgl           @191
   _h_maybe_switch_to_hpgl             @192
   _h_paint_path                       @193
   _h_paint_paths                      @194
   _h_paint_point                      @195
   _h_paint_text_string                @196
   _h_set_attributes                   @197
   _h_set_fill_color                   @198
   _h_set_font                         @199
   _h_set_pen_color                    @200
   _h_set_position                     @201
   _h_terminate                        @202
   _hershey_accented_char_info         @203
   _hershey_font_info                  @204
   _hershey_typeface_info              @205
   _hpgl2_maybe_update_font            @206
   _hpgl_maybe_update_font             @207
   _hpgl_pseudocolor                   @208
   _hpgl_shaded_pseudocolor            @209
   _i_begin_page                       @210
   _i_default_plotter                  @211
   _i_delete_image                     @212
   _i_draw_elliptic_arc                @213
   _i_draw_elliptic_arc_2              @214
   _i_draw_elliptic_arc_internal       @215
   _i_end_page                         @216
   _i_erase_page                       @217
   _i_initialize                       @218
   _i_new_color_index                  @219
   _i_new_image                        @220
   _i_paint_path                       @221
   _i_paint_paths                      @222
   _i_paint_point                      @223
   _i_scan_pixel                       @224
   _i_set_bg_color                     @225
   _i_set_fill_color                   @226
   _i_set_pen_color                    @227
   _i_start_scan                       @228
   _i_terminate                        @229
   _i_write_gif_header                 @230
   _i_write_gif_image                  @231
   _i_write_gif_trailer                @232
   _i_write_short_int                  @233
   _idraw_brush_pattern                @234
   _idraw_stdcolornames                @235
   _idraw_stdcolors                    @236
   _idraw_stdshadings                  @237
   _kermit_bgcolor_escapes             @238
   _kermit_fgcolor_escapes             @239
   _kermit_stdcolors                   @240
   _known_params                       @241
   _libplot_color_to_svg_color         @242
   _line_styles                        @243
   _m_begin_page                       @244
   _m_default_plotter                  @245
   _m_emit_float                       @246
   _m_emit_integer                     @247
   _m_emit_op_code                     @248
   _m_emit_string                      @249
   _m_emit_terminator                  @250
   _m_end_page                         @251
   _m_erase_page                       @252
   _m_initialize                       @253
   _m_maybe_prepaint_segments          @254
   _m_paint_marker                     @255
   _m_paint_path                       @256
   _m_paint_path_internal              @257
   _m_paint_paths                      @258
   _m_paint_point                      @259
   _m_paint_text_string_with_escapes   @260
   _m_path_is_flushable                @261
   _m_set_attributes                   @262
   _m_terminate                        @263
   _matrix_inverse                     @264
   _matrix_norm                        @265
   _matrix_product                     @266
   _matrix_sing_vals                   @267
   _maybe_output_image                 @268
   _maybe_replace_arc                  @269
   _maybe_switch_from_hpgl             @270
   _maybe_switch_to_hpgl               @271
   _merge_paths                        @272
   _miAddSpansToPaintedSet             @273
   _miClearPaintedSet                  @274
   _miCopyCanvas                       @275
   _miCopyGC                           @276
   _miCopyPaintedSetToCanvas           @277
   _miDeleteCanvas                     @278
   _miDeleteEllipseCache               @279
   _miDeleteGC                         @280
   _miDeletePaintedSet                 @281
   _miDrawArcs_r                       @282
   _miDrawLines                        @283
   _miDrawPoints                       @284
   _miDrawRectangles                   @285
   _miFillArcs                         @286
   _miFillPolygon                      @287
   _miFillRectangles                   @288
   _miNewCanvas                        @289
   _miNewEllipseCache                  @290
   _miNewGC                            @291
   _miNewPaintedSet                    @292
   _miSetCanvasStipple                 @293
   _miSetCanvasTexture                 @294
   _miSetGCAttrib                      @295
   _miSetGCAttribs                     @296
   _miSetGCDashes                      @297
   _miSetGCMiterLimit                  @298
   _miSetGCPixels                      @299
   _miSetPixelMerge2                   @300
   _miSetPixelMerge3                   @301
   _mi_libxmi_ver                      @302
   _n_default_plotter                  @303
   _n_initialize                       @304
   _n_maybe_output_image               @305
   _n_terminate                        @306
   _n_write_pbm                        @307
   _n_write_pgm                        @308
   _n_write_pnm                        @309
   _n_write_ppm                        @310
   _new_outbuf                         @311
   _new_plPath                         @312
   _occidental_hershey_glyphs          @313
   _old_api_global_plotter_params      @314
   _oriental_hershey_glyphs            @315
   _p_begin_page                       @316
   _p_compute_idraw_bgcolor            @317
   _p_default_plotter                  @318
   _p_emit_common_attributes           @319
   _p_end_page                         @320
   _p_erase_page                       @321
   _p_fellipse_internal                @322
   _p_initialize                       @323
     _p_paint_path                     @1
   _p_paint_paths                      @325
   _p_paint_point                      @326
   _p_paint_text_string                @327
   _p_set_fill_color                   @328
   _p_set_pen_color                    @329
   _p_terminate                        @330
   _parse_pen_string                   @331
   _pcl_font_info                      @332
   _pcl_typeface_info                  @333
   _plot_xcalloc                       @334
   _plot_xmalloc                       @335
   _plot_xrealloc                      @336
   _plotters                           @337
   _plotters_len                       @338
   _ps_cap_style                       @339
   _ps_font_info                       @340
   _ps_font_to_cgm_font_id             @341
   _ps_join_style                      @342
   _ps_typeface_info                   @343
   _q_default_plotter                  @344
   _q_initialize                       @345
   _q_maybe_switch_from_hpgl           @346
   _q_maybe_switch_to_hpgl             @347
   _q_terminate                        @348
   _r_begin_page                       @349
   _r_default_plotter                  @350
   _r_end_page                         @351
   _r_erase_page                       @352
   _r_initialize                       @353
   _r_maybe_prepaint_segments          @354
   _r_paint_path                       @355
   _r_paint_paths                      @356
   _r_paint_point                      @357
   _r_path_is_flushable                @358
   _r_set_attributes                   @359
   _r_set_bg_color                     @360
   _r_set_fill_color                   @361
   _r_set_pen_color                    @362
   _r_terminate                        @363
   _regis_color_chars                  @364
   _regis_move                         @365
   _regis_stdcolors                    @366
   _render_non_hershey_string          @367
   _render_simple_string               @368
   _reset_outbuf                       @369
   _reset_plPath                       @370
   _rle_do_pixel                       @371
   _rle_init                           @372
   _rle_terminate                      @373
   _s_begin_page                       @374
   _s_default_plotter                  @375
   _s_end_page                         @376
   _s_erase_page                       @377
   _s_initialize                       @378
   _s_paint_path                       @379
   _s_paint_paths                      @380
   _s_paint_point                      @381
   _s_paint_text_string                @382
   _s_set_matrix                       @383
   _s_terminate                        @384
   _set_bezier2_bbox                   @385
   _set_bezier3_bbox                   @386
   _set_common_mi_attributes           @387
   _set_ellipse_bbox                   @388
   _set_font                           @389
   _set_hpgl_fill_type                 @390
   _set_hpgl_pen                       @391
   _set_hpgl_pen_type                  @392
   _set_line_end_bbox                  @393
   _set_line_join_bbox                 @394
   _set_page_type                      @395
   _setplparam                         @396
   _stick_font_info                    @397
   _stick_kerning_tables               @398
   _stick_spacing_tables               @399
   _stick_typeface_info                @400
   _string_to_color                    @401
   _svg_colornames                     @402
   _t_begin_page                       @403
   _t_default_plotter                  @404
   _t_end_page                         @405
   _t_erase_page                       @406
   _t_initialize                       @407
   _t_maybe_prepaint_segments          @408
   _t_paint_point                      @409
   _t_path_is_flushable                @410
   _t_set_attributes                   @411
   _t_set_bg_color                     @412
   _t_set_pen_color                    @413
   _t_terminate                        @414
   _tek_mode                           @415
   _tek_move                           @416
   _tek_vector                         @417
   _tek_vector_compressed              @418
   _truecenter                         @419
   _update_bbox                        @420
   _update_buffer                      @421
   _update_buffer_by_added_bytes       @422
   _variable_width_katakana_spacings   @423
   _variable_width_spacings            @424
   _vscale                             @425
   _write_byte                         @426
   _write_bytes                        @427
   _write_string                       @428
   _xatan2                             @429
   _z_default_plotter                  @430
   _z_initialize                       @431
   _z_maybe_output_image               @432
   _z_terminate                        @433
   libplot_error_handler               @434
   libplot_warning_handler             @435
   pl_alabel                           @436
   pl_alabel_r                         @437
   pl_arc                              @438
   pl_arc_r                            @439
   pl_arcrel                           @440
   pl_arcrel_r                         @441
   pl_bezier2                          @442
   pl_bezier2_r                        @443
   pl_bezier2rel                       @444
   pl_bezier2rel_r                     @445
   pl_bezier3                          @446
   pl_bezier3_r                        @447
   pl_bezier3rel                       @448
   pl_bezier3rel_r                     @449
   pl_bgcolor                          @450
   pl_bgcolor_r                        @451
   pl_bgcolorname                      @452
   pl_bgcolorname_r                    @453
   pl_box                              @454
   pl_box_r                            @455
   pl_boxrel                           @456
   pl_boxrel_r                         @457
   pl_capmod                           @458
   pl_capmod_r                         @459
   pl_circle                           @460
   pl_circle_r                         @461
   pl_circlerel                        @462
   pl_circlerel_r                      @463
   pl_closepath                        @464
   pl_closepath_r                      @465
   pl_closepl                          @466
   pl_closepl_r                        @467
   pl_color                            @468
   pl_color_r                          @469
   pl_colorname                        @470
   pl_colorname_r                      @471
   pl_cont                             @472
   pl_cont_r                           @473
   pl_contrel                          @474
   pl_contrel_r                        @475
   pl_copyplparams                     @476
   pl_deletepl                         @477
   pl_deletepl_r                       @478
   pl_deleteplparams                   @479
   pl_ellarc                           @480
   pl_ellarc_r                         @481
   pl_ellarcrel                        @482
   pl_ellarcrel_r                      @483
   pl_ellipse                          @484
   pl_ellipse_r                        @485
   pl_ellipserel                       @486
   pl_ellipserel_r                     @487
   pl_endpath                          @488
   pl_endpath_r                        @489
   pl_endsubpath                       @490
   pl_endsubpath_r                     @491
   pl_erase                            @492
   pl_erase_r                          @493
   pl_farc                             @494
   pl_farc_r                           @495
   pl_farcrel                          @496
   pl_farcrel_r                        @497
   pl_fbezier2                         @498
   pl_fbezier2_r                       @499
   pl_fbezier2rel                      @500
   pl_fbezier2rel_r                    @501
   pl_fbezier3                         @502
   pl_fbezier3_r                       @503
   pl_fbezier3rel                      @504
   pl_fbezier3rel_r                    @505
   pl_fbox                             @506
   pl_fbox_r                           @507
   pl_fboxrel                          @508
   pl_fboxrel_r                        @509
   pl_fcircle                          @510
   pl_fcircle_r                        @511
   pl_fcirclerel                       @512
   pl_fcirclerel_r                     @513
   pl_fconcat                          @514
   pl_fconcat_r                        @515
   pl_fcont                            @516
   pl_fcont_r                          @517
   pl_fcontrel                         @518
   pl_fcontrel_r                       @519
   pl_fellarc                          @520
   pl_fellarc_r                        @521
   pl_fellarcrel                       @522
   pl_fellarcrel_r                     @523
   pl_fellipse                         @524
   pl_fellipse_r                       @525
   pl_fellipserel                      @526
   pl_fellipserel_r                    @527
   pl_ffontname                        @528
   pl_ffontname_r                      @529
   pl_ffontsize                        @530
   pl_ffontsize_r                      @531
   pl_fillcolor                        @532
   pl_fillcolor_r                      @533
   pl_fillcolorname                    @534
   pl_fillcolorname_r                  @535
   pl_fillmod                          @536
   pl_fillmod_r                        @537
   pl_filltype                         @538
   pl_filltype_r                       @539
   pl_flabelwidth                      @540
   pl_flabelwidth_r                    @541
   pl_fline                            @542
   pl_fline_r                          @543
   pl_flinedash                        @544
   pl_flinedash_r                      @545
   pl_flinerel                         @546
   pl_flinerel_r                       @547
   pl_flinewidth                       @548
   pl_flinewidth_r                     @549
   pl_flushpl                          @550
   pl_flushpl_r                        @551
   pl_fmarker                          @552
   pl_fmarker_r                        @553
   pl_fmarkerrel                       @554
   pl_fmarkerrel_r                     @555
   pl_fmiterlimit                      @556
   pl_fmiterlimit_r                    @557
   pl_fmove                            @558
   pl_fmove_r                          @559
   pl_fmoverel                         @560
   pl_fmoverel_r                       @561
   pl_fontname                         @562
   pl_fontname_r                       @563
   pl_fontsize                         @564
   pl_fontsize_r                       @565
   pl_fpoint                           @566
   pl_fpoint_r                         @567
   pl_fpointrel                        @568
   pl_fpointrel_r                      @569
   pl_frotate                          @570
   pl_frotate_r                        @571
   pl_fscale                           @572
   pl_fscale_r                         @573
   pl_fsetmatrix                       @574
   pl_fsetmatrix_r                     @575
   pl_fspace                           @576
   pl_fspace2                          @577
   pl_fspace2_r                        @578
   pl_fspace_r                         @579
   pl_ftextangle                       @580
   pl_ftextangle_r                     @581
   pl_ftranslate                       @582
   pl_ftranslate_r                     @583
   pl_get_hershey_font_info            @584
   pl_get_pcl_font_info                @585
   pl_get_ps_font_info                 @586
   pl_get_stick_font_info              @587
   pl_havecap                          @588
   pl_havecap_r                        @589
   pl_joinmod                          @590
   pl_joinmod_r                        @591
   pl_label                            @592
   pl_label_r                          @593
   pl_labelwidth                       @594
   pl_labelwidth_r                     @595
   pl_libplot_ver                      @596
   pl_line                             @597
   pl_line_r                           @598
   pl_linedash                         @599
   pl_linedash_r                       @600
   pl_linemod                          @601
   pl_linemod_r                        @602
   pl_linerel                          @603
   pl_linerel_r                        @604
   pl_linewidth                        @605
   pl_linewidth_r                      @606
   pl_marker                           @607
   pl_marker_r                         @608
   pl_markerrel                        @609
   pl_markerrel_r                      @610
   pl_move                             @611
   pl_move_r                           @612
   pl_moverel                          @613
   pl_moverel_r                        @614
   pl_newpl                            @615
   pl_newpl_r                          @616
   pl_newplparams                      @617
   pl_openpl                           @618
   pl_openpl_r                         @619
   pl_orientation                      @620
   pl_orientation_r                    @621
   pl_outfile                          @622
   pl_outfile_r                        @623
   pl_parampl                          @624
   pl_pencolor                         @625
   pl_pencolor_r                       @626
   pl_pencolorname                     @627
   pl_pencolorname_r                   @628
   pl_pentype                          @629
   pl_pentype_r                        @630
   pl_point                            @631
   pl_point_r                          @632
   pl_pointrel                         @633
   pl_pointrel_r                       @634
   pl_restorestate                     @635
   pl_restorestate_r                   @636
   pl_savestate                        @637
   pl_savestate_r                      @638
   pl_selectpl                         @639
   pl_setplparam                       @640
   pl_space                            @641
   pl_space2                           @642
   pl_space2_r                         @643
   pl_space_r                          @644
   pl_textangle                        @645
   pl_textangle_r                      @646

; end of file
